# frozen_string_literal: true

require 'msgpack'

module CommunityTheatre
  class SplicingSpooler < DisposableSpooler
    def initialize(dest, page_size: 100)
      super(dest, page_size: page_size)
    end

    def spool!(enumerable)
      pages = existing_pages
      # for convenience's sake, rebuild the last page now
      starting_page = [pages.size - 1, 0].max
      skip_number = starting_page * @page_size

      enumerable.seek_to!(skip_number, last_entry_of_second_last_page)

      enumerable.each_slice(@page_size).each_with_index do |page, index|
        file_for_page(index + starting_page) do |file|
          packer = MessagePack::Packer.new(file)
          packer.write_array_header(page.size)

          page.each { |item| packer.write(item) }

          packer.flush
        end
      end
    end

    private

    def last_entry_of_second_last_page
      pages = existing_pages

      return if pages.size < 2

      file_for_page(pages[-2], 'r') do |f|
        page = MessagePack::Unpacker.new(f).unpack

        page.last
      end
    end

  end
end
